import os
import tempfile
import shutil
import json

import pytest


@pytest.fixture(scope='function')
def tenants(transactional_db, request, settings):
    from hobo.multitenant.models import Tenant
    base = tempfile.mkdtemp('combo-tenant-base')
    settings.TENANT_BASE = base

    @pytest.mark.django_db
    def make_tenant(name):
        tenant_dir = os.path.join(base, name)
        os.mkdir(tenant_dir)
        with open(os.path.join(tenant_dir, 'unsecure'), 'w') as fd:
            fd.write('1')
        with open(os.path.join(tenant_dir, 'settings.json'), 'w') as fd:
            json.dump({
                'HOBO_TEST_VARIABLE': name,
                'UPDATE_ME.update': {'y': 2},
                'EXTEND_ME.extend': [2],
            }, fd)
        with open(os.path.join(tenant_dir, 'hobo.json'), 'w') as fd:
            json.dump({
                'variables': {
                    'hobo_test_variable': True,
                    'other_variable': 'foo',
                    'theme': 'publik',
                },
                'services': [
                    {'slug': 'test',
                     'title': 'Test',
                     'service-id': 'welco',
                     'this': True,
                     'secret_key': '12345',
                     'base_url': 'http://%s' % name,
                     'saml-sp-metadata-url': 'http://%s/saml/metadata' % name,
                     'variables': {
                         'other_variable': 'bar',
                     }
                    },
                    {'slug': 'slug-with-hyphen',
                     'title': 'Hyphen',
                     'base_url': 'http://slug-with-hyphen.example.net'
                    },
                    {'slug': 'other',
                     'title': 'Other',
                     'secret_key': 'abcde',
                     'service-id': 'authentic',
                     'base_url': 'http://other.example.net'},
                    ]}, fd)
        t = Tenant(domain_url=name,
                      schema_name=name.replace('-', '_').replace('.', '_'))
        t.create_schema()
        return t
    tenants = [make_tenant('tenant1.example.net'), make_tenant('tenant2.example.net')]
    def fin():
        from django.db import connection
        from hobo.multitenant.middleware import TenantMiddleware
        connection.set_schema_to_public()
        for t in TenantMiddleware.get_tenants():
            t.delete(True)
        shutil.rmtree(base)
    request.addfinalizer(fin)
    return tenants
