import logging
from hobo.logger import RequestContextFilter
from tenant_schemas.utils import tenant_context

from django.contrib.auth.models import User


def test_request_context_filter(caplog, settings, tenants, client):
    root_logger = logging.getLogger()
    assert len(root_logger.handlers) == 1
    root_logger.handlers[0].addFilter(RequestContextFilter())
    for tenant in tenants:
        with tenant_context(tenant):
            user = User.objects.create(first_name='John', last_name='Doe', username='john.doe',
                                       email='jodn.doe@example.com')
            user.set_password('john.doe')
            user.save()
            user.saml_identifiers.create(name_id='ab' * 16, issuer='https://idp.example.com')

    for tenant in tenants:
        settings.ALLOWED_HOSTS.append(tenant.domain_url)
        with tenant_context(tenant):
            client.login(username='john.doe', password='john.doe')
        client.get('/', SERVER_NAME=tenant.domain_url,
                   HTTP_X_FORWARDED_FOR='99.99.99.99, 127.0.0.1')
    records = [record for record in caplog.records if record.levelno >= logging.INFO]
    assert len(records) == 2
    for tenant, record in zip(tenants, records):
        assert record.ip == '99.99.99.99'
        assert record.tenant == tenant.domain_url
        assert record.path == '/'
        assert record.request_id.startswith('r:')
        assert record.user == user.username
        assert record.user_email == user.email
        assert record.user_name == user.username
        assert record.user_display_name == 'John Doe'
        assert record.user_uuid == 'ab' * 16
        assert record.application == 'fake-agent'
